#! /usr/bin/python
# -*- Mode: Python -*-
# vi:si:et:sw=4:sts=4:ts=4

from twisted.spread import pb
from twisted.internet import reactor
from twisted.cred import credentials
from twisted.cred import checkers, portal

class Manager:
    pass

class Dispatcher:
    __implements__ = portal.IRealm
    def __init__(self, manager):
        print "manager: creating dispatcher"
        self.manager = manager

    def requestAvatar(self, avatarID, mind, *interfaces):
        print "manager: dispatcher: avatar being requested: %s" % avatarID
        if avatarID == "admin":
            avatar = AdminAvatar(self.manager)
        else:
            avatar = ComponentAvatar(self.manager)

        reactor.callLater(0, avatar.attached, mind)
        return (pb.IPerspective, avatar, avatar.detached)
    
class AdminAvatar(pb.Avatar):
    def __init__(self, manager):
        self.manager = manager
        manager.admin = self

    def attached(self, mind):
        print
        print "manager: AdminAvatar: mind %s attached" % mind
        self.mind = mind

    def detached(self):
        print "manager: AdminAvatar: mind detached"
        print
        pass

    def errback(self, failure):
        print "manager: admin asked for failure"
        print "manager: failure: %r" % failure
        print "manager: message: %s" % failure.getErrorMessage()

        # import code; code.interact(local=locals())

        # all of the lines below get us
        # Traceback from remote host -- Failure: twisted.spread.pb.Error: I failed admin
        
        #return pb.Error("forwarding failure from component to admin")
        #return failure

        # this raise will get us a complete traceback on stderr
        # raise pb.Error("forwarding failure from component to admin")

        # turning the CopiedFailure into a CopyableFailure again gives us
        # Failure: twisted.spread.pb.Error: I failed admin
        # which comes from twisted.python.failure.Failure.printTraceback()
        # copyable = pb.failure2Copyable(failure)
        # copyable.printTraceback()
        # return copyable

        # monkeypatching the printTraceback method raises InsecureJelly
        # copyable = pb.failure2Copyable(failure)
        # copyable.printTraceback = lambda *args, **kwargs: None
        # return copyable

        return failure

    def perspective_succeed(self):
        print "manager: AdminAvatar: admin asked to succeed"
        print "manager: forwarding succeed call to Component"
        if not self.manager.component:
            print "manager: ERROR: component not logged in"
            return
        return self.manager.component.mind.callRemote("succeed", "admin")

    def perspective_fail(self):
        print "manager: AdminAvatar: admin asked to fail"
        print "manager: forwarding fail call to Component"
        if not self.manager.component:
            print "manager: ERROR: component not logged in"
            return
        d = self.manager.component.mind.callRemote("fail", "admin")
        d.addErrback(self.errback)
        return d

class ComponentAvatar(pb.Avatar):
    def __init__(self, manager):
        self.manager = manager
        manager.component = self

    def attached(self, mind):
        print
        print "manager: ComponentAvatar: mind %s attached" % mind
        self.mind = mind

    def detached(self):
        print "manager: ComponentAvatar: mind detached"
        print
        pass

    def perspective_ready(self):
        print "manager: I am ready for the component.  Can it succeed if I ask it to ?"
        d = self.mind.callRemote("succeed", "manager")
        d.addCallback(self.fail)
        return d
        
    def fail(self, somearg):
        print "manager: the component succeeded"
        print "manager: and can it fail ?"
        d = self.mind.callRemote("fail", "manager")
        d.addErrback(self.errback)
        return d

    def errback(self, failure):
        print "manager: I asked for failure"
        print "manager: failure: %r" % failure
        print "manager: message: %s" % failure.getErrorMessage()

def startManager():
    m = Manager()
    p = portal.Portal(Dispatcher(m))
    p.registerChecker(checkers.InMemoryUsernamePasswordDatabaseDontUse(admin="pass",component="pass"))

    factory = pb.PBServerFactory(p)
    factory.unsafeTracebacks = 1
    reactor.listenTCP(8900, factory)

if __name__ == '__main__':
    startManager()
    print "main: running reactor"
    reactor.run()
